package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.dto.SubjectiveQuestionDTO;
import cn.com.qmth.stmms.admin.thread.DataSyncThread;
import cn.com.qmth.stmms.admin.utils.HttpUtil;
import cn.com.qmth.stmms.biz.exam.model.DataSync;
import cn.com.qmth.stmms.biz.exam.service.DataSyncService;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.common.auth.annotation.RoleRequire;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.utils.ExportExcel;

@Controller
@RequestMapping("/admin/exam/data/sync")
public class DataSyncController extends BaseExamController {

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private LockService lockService;

    @Autowired
    private ExamService examService;

    @Autowired
    private DataSyncService dataSyncService;

    @Value("${qmth.examcloud.host}")
    private String host;

    @Value("${qmth.examcloud.port}")
    private String port;

    @Value("${qmth.examcloud.rootOrgId}")
    private String rootOrgId;

    @Value("${qmth.examcloud.appId}")
    private String appId;

    @Value("${qmth.examcloud.secretKey}")
    private String secretKey;

    @Value("${qmth.examcloud.subject.uri}")
    private String subjectUri;

    @Value("${qmth.examcloud.student.uri}")
    private String studentUri;

    @Value("${qmth.examcloud.pageSize}")
    private Integer pageSize;

    @Value("${file.root}")
    private String baseDir;

    @RequestMapping
    public ModelAndView index(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        ModelAndView view = new ModelAndView("modules/exam/dataSync");
        view.addObject("running", lockService.isLocked(LockType.DATA_SYNC, examId));
        view.addObject("host", host);
        view.addObject("port", port);
        view.addObject("rootOrgId", rootOrgId);
        view.addObject("subjectUri", subjectUri);
        view.addObject("studentUri", studentUri);
        return view;
    }

    @RequestMapping(value = "/start", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView sync1(HttpServletRequest request, DataSync dataSync) {
        int examId = getSessionExamId(request);
        if (lockService.trylock(LockType.DATA_SYNC, examId)) {
            DataSyncThread thread = new DataSyncThread(dataSync, secretKey, appId, pageSize, baseDir, lockService,
                    dataSyncService, examService, studentService);
            taskExecutor.submit(thread);
        }
        return new ModelAndView("redirect:/admin/exam/dataSync");
    }

    @RequestMapping(value = "/export", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView exportFile(String url, HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);
        DataSync sync = dataSyncService.findByExamId(examId);
        List<SubjectiveQuestionDTO> list = getPaperStruct(url, sync.getCloudExamId());
        try {
            new ExportExcel("主观题数据", SubjectiveQuestionDTO.class, 2).setDataList(list).write(response, "主观题数据.xlsx")
                    .dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出数据失败！失败信息：" + e.getMessage());
        }
        return new ModelAndView("redirect:/admin/exam");
    }

    private List<SubjectiveQuestionDTO> getPaperStruct(String url, Long cloudExamId) {
        List<SubjectiveQuestionDTO> list = new ArrayList<SubjectiveQuestionDTO>();

        HttpUtil subjectHttp = new HttpUtil(url, secretKey, appId, rootOrgId);
        JSONObject datas = new JSONObject();
        datas.accumulate("examId", cloudExamId);
        String subjectResult = subjectHttp.httpAction(null, datas.toString());
        JSONObject subjectJson = JSONObject.fromObject(subjectResult);
        JSONArray subjectArray = subjectJson.getJSONArray("paperStructList");
        for (int i = 0; i < subjectArray.size(); i++) {
            JSONObject subject = subjectArray.getJSONObject(i);
            JSONArray questionArray = subject.getJSONArray("questions");
            for (int j = 0; j < questionArray.size(); j++) {
                JSONObject question = questionArray.getJSONObject(i);
                SubjectiveQuestionDTO questionDTO = new SubjectiveQuestionDTO();
                questionDTO.setSubjectCode(subject.getString("subjectCode"));
                questionDTO.setSubjectName(subject.getString("subjectName"));
                questionDTO.setMainNumber(question.getInt("mainNumber"));
                questionDTO.setTitle(question.getString("mainTitle"));
                questionDTO.setSubNumber(question.getInt("subNumber"));
                questionDTO.setTotalScore(question.getDouble("totalScore"));
                list.add(questionDTO);
            }
        }
        return list;
    }
}
